package com.ff.apimngr.controller;

import com.ff.apimngr.dao.ApiMapper;
import com.ff.apimngr.dao.UserMapper;
import com.ff.apimngr.entity.Api;
import com.ff.apimngr.entity.ApiAuthApply;
import com.ff.apimngr.entity.ApiCollect;
import com.ff.apimngr.entity.User;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.ApiAuthApplyService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/api/apiAuthApply")
public class ApiAuthApplyController {

    @Autowired
    private ApiAuthApplyService apiAuthApplyService;

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private ApiMapper apiMapper;

    @ApiOperation(value = "权限申请新增", notes = "权限申请",tags = "权限申请")
    @PostMapping(value = "/insert")
    public Map<String, Object> insert(@ApiParam(value = "接口id", required = true) @RequestParam Integer apiId,
                                         @ApiParam(value = "申请用途", required = false) @RequestParam String applyUse,
                                         HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }
        Api api = apiMapper.findById(apiId);
        if(api==null)
        {
            result.put("code", 1002);
            result.put("message", "接口信息不存在");
            return result;
        }


        ApiAuthApply apiAuthApply  = new ApiAuthApply();
        apiAuthApply.setApiId(apiId);
        apiAuthApply.setUserId(user.getId());
        apiAuthApply.setApplyUse(applyUse);
        apiAuthApply.setStatus(0);
        Date date = Calendar.getInstance().getTime();
        apiAuthApply.setApplyTime(date);

        apiAuthApplyService.insertApiAuthApply(apiAuthApply);
        result.put("code", 10000);
        result.put("message", "申请成功");
        return result;
    }

    @ApiOperation(value = "权限申请审核", notes = "权限申请",tags = "权限申请")
    @PostMapping(value = "/update")
    public Map<String, Object> update(@ApiParam(value = "申请id", required = true) @RequestParam Integer id,
                                      @ApiParam(value = "审核内容:0 不同意、1 已同意", required = true) @RequestParam Integer examineContent,
                                      @ApiParam(value = "申请用途", required = true) @RequestParam String disagreeReason,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }

        ApiAuthApply apiAuthApply  = new ApiAuthApply();
        apiAuthApply.setId(id);
        apiAuthApply.setExamineUserId(user.getId());
        apiAuthApply.setExamineContent(examineContent);
        apiAuthApply.setDisagreeReason(disagreeReason);
        apiAuthApply.setStatus(1);
        Date date = Calendar.getInstance().getTime();
        apiAuthApply.setExamineTime(date);

        apiAuthApplyService.updateApiAuthApply(apiAuthApply);
        result.put("code", 10000);
        result.put("message", "审核成功");
        return result;
    }



    @ApiOperation(value = "权限申请列表", notes = "权限申请",tags = "权限申请")
    @PostMapping(value = "/applyList")
    public Map<String, Object> applyList(@RequestParam(value = "apiName", required = false) String apiName,
                                         @RequestParam(value = "apiType", required = false) Integer apiType,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }
        result.put("data",apiAuthApplyService.getApplyList(user.getId(),apiName,apiType));
        result.put("code", 10000);
        result.put("message", "查询权限申请列表成功");
        return result;
    }

    @ApiOperation(value = "权限审核列表", notes = "权限申请",tags = "权限申请")
    @PostMapping(value = "/examineList")
    public Map<String, Object> examineList(@RequestParam(value = "status", required = false)  Integer status,
                                      @RequestParam(value = "userName", required = false)  String userName,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        result.put("data",apiAuthApplyService.getExamineList(status,userName));
        result.put("code", 10000);
        result.put("message", "查询权限审核列表成功");
        return result;
    }


    @ApiOperation(value = "权限申请详情", notes = "权限申请",tags = "权限申请")
    @GetMapping(value = "/getDetail")
    public Map<String, Object> getDetail(@ApiParam(value = "申请id", required = true) @RequestParam Integer id,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        result.put("data",apiAuthApplyService.selectById(id));
        result.put("code", 10000);
        result.put("message", "查询权限申请详情成功");
        return result;
    }




}
