package com.ff.apimngr.controller;


import com.ff.apimngr.dao.ApiCollectMapper;
import com.ff.apimngr.dao.ApiMapper;
import com.ff.apimngr.dao.UserMapper;
import com.ff.apimngr.entity.Api;
import com.ff.apimngr.entity.ApiCollect;
import com.ff.apimngr.entity.ApiCollectList;
import com.ff.apimngr.entity.User;
import com.ff.apimngr.jwt.JwtHelper;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;


//@io.swagger.annotations.Api(tags = "接口收藏",position = 3, description = "")
@RestController
@RequestMapping("/api/apiCollect")
public class ApiCollectController
{
    @Resource
    ApiCollectMapper apiCollectMapper;
    @Resource
    ApiMapper apiMapper;
    @Resource
    UserMapper userMapper;


    @ApiOperation(value = "接口收藏新增", notes = "接口收藏",tags = "接口收藏")
    @PostMapping(value = "/insert")
    public Map<String, Object> insertApi(@ApiParam(value = "接口id", required = true) @RequestParam Integer apiId,
//                                         @ApiParam(value = "是否收藏接口：1收藏、0取消", required = false) @RequestParam Integer isCollect,
                                         HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }
        Api api = apiMapper.findById(apiId);
        if(api==null)
        {
            result.put("code", 1002);
            result.put("message", "接口信息不存在");
            return result;
        }

//        if(isCollect==null)
//        {
//            result.put("code", 1003);
//            result.put("message", "是否收藏状态不能为空");
//            return result;
//        }
        ApiCollect apiCollect  = new ApiCollect();
        apiCollect.setApiId(apiId);
        apiCollect.setUserId(user.getId());
        apiCollect.setIsCollect(1);

        Date  date = Calendar.getInstance().getTime();
        apiCollect.setCreateTime(date);
        apiCollect.setModifyTime(date);

        Integer num = apiCollectMapper.insert(apiCollect);



        result.put("code", 10000);
        result.put("message", "创建成功");
        return result;
    }

    @ApiOperation(value = "接口收藏取消", notes = "接口收藏",tags = "接口收藏")
    @PostMapping(value = "/cancel")
    public Map<String, Object> cancelApi(
                                         @ApiParam(value = "收藏记录id", required = true) @RequestParam Integer acId,
                                         HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }

        ApiCollect apiCollect = apiCollectMapper.selectById(acId);
        if(apiCollect==null)
        {
            result.put("code", 1002);
            result.put("message", "接口收藏记录不存在");
            return result;
        }
        if(apiCollect.getUserId()==user.getId())
        {
            result.put("code", 1004);
            result.put("message", "接口收藏用户与当前登录用户不匹配");
            return result;
        }

        Date  date = Calendar.getInstance().getTime();
        apiCollect.setModifyTime(date);

        Integer num = apiCollectMapper.update(acId,0);
        result.put("code", 10000);
        result.put("message", "取消收藏成功");
        return result;
    }

    @ApiOperation(value = "接口收藏列表", notes = "接口收藏",tags = "接口收藏")
    @PostMapping(value = "/list")
    public Map<String, Object> listApi(HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }

        List<ApiCollectList> list = apiCollectMapper.apiCollectList(user.getId());
        result.put("code", 10000);
        result.put("data", list);
        result.put("message", "获取成功");
        return result;
    }

}
