package com.ff.apimngr.controller;

import com.ff.apimngr.dao.ApiMapper;
import com.ff.apimngr.dao.UserMapper;
import com.ff.apimngr.entity.Api;
import com.ff.apimngr.entity.ApiAuthApply;
import com.ff.apimngr.entity.ApiNeedApply;
import com.ff.apimngr.entity.User;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.ApiNeedApplyService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/api/apiNeedApply")
public class ApiNeedApplyController {


    @Autowired
    private ApiNeedApplyService apiNeedApplyService;

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private ApiMapper apiMapper;

    @ApiOperation(value = "需求申请新增", notes = "需求申请",tags = "需求申请")
    @PostMapping(value = "/insert")
    public Map<String, Object> insert(@ApiParam(value = "接口id", required = true) @RequestParam Integer apiId,
                                      @ApiParam(value = "联系方式", required = true) @RequestParam String mobile,
                                      @ApiParam(value = "字段信息", required = true) @RequestParam String fieldJson,
                                      @ApiParam(value = "需求描述", required = true) @RequestParam String description,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }
        Api api = apiMapper.findById(apiId);
        if(api==null)
        {
            result.put("code", 1002);
            result.put("message", "接口信息不存在");
            return result;
        }


        ApiNeedApply apiNeedApply  = new ApiNeedApply();
        apiNeedApply.setApiId(apiId);
        apiNeedApply.setUserId(user.getId());
        apiNeedApply.setMobile(mobile);
        apiNeedApply.setFieldJson(fieldJson);
        apiNeedApply.setDescription(description);
        apiNeedApply.setStatus(0);
        Date date = Calendar.getInstance().getTime();
        apiNeedApply.setApplyTime(date);
        apiNeedApplyService.insert(apiNeedApply);
        result.put("code", 10000);
        result.put("message", "申请成功");
        return result;
    }

    @ApiOperation(value = "需求申请审核", notes = "需求申请",tags = "需求申请")
    @PostMapping(value = "/update")
    public Map<String, Object> update(@RequestParam(value = "id", required = true) Integer id,
                                      @RequestParam(value = "status", required = true) Integer status,
                                      @RequestParam(value = "disagreeReason", required = false)  String disagreeReason,
                                      HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }

        ApiNeedApply apiNeedApply  = new ApiNeedApply();
        apiNeedApply.setId(id);
        apiNeedApply.setExamineUserId(user.getId());
        apiNeedApply.setDisagreeReason(disagreeReason);
        apiNeedApply.setStatus(status);
        Date date = Calendar.getInstance().getTime();
        apiNeedApply.setExamineTime(date);

        apiNeedApplyService.update(apiNeedApply);
        result.put("code", 10000);
        result.put("message", "审核成功");
        return result;
    }



    @ApiOperation(value = "需求申请列表", notes = "需求申请",tags = "需求申请")
    @PostMapping(value = "/applyList")
    public Map<String, Object> applyList(@RequestParam(value = "apiName", required = false) String apiName,
                                         @RequestParam(value = "apiType", required = false) Integer apiType,
                                         HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userMapper.findUserByAccount(account);
        if (null == user) {
            result.put("code", 1001);
            result.put("message", "用户信息不存在");
            return result;
        }
        result.put("data",apiNeedApplyService.getApplyList(user.getId(),apiName,apiType));
        result.put("code", 10000);
        result.put("message", "查询需求申请列表成功");
        return result;
    }

    @ApiOperation(value = "需求审核列表", notes = "需求申请",tags = "需求申请")
    @PostMapping(value = "/examineList")
    public Map<String, Object> examineList(@RequestParam(value = "status", required = false) Integer status,
                                           @RequestParam(value = "userName", required = false)  String userName,
                                           HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        result.put("data",apiNeedApplyService.getExamineList(status,userName));
        result.put("code", 10000);
        result.put("message", "查询需求审核列表成功");
        return result;
    }


    @ApiOperation(value = "需求申请详情", notes = "需求申请",tags = "需求申请")
    @GetMapping(value = "/getDetail")
    public Map<String, Object> getDetail(@ApiParam(value = "申请id", required = true) @RequestParam Integer id,
                                         HttpServletRequest request)
    {
        Map<String, Object> result = new HashMap<>();
        result.put("data",apiNeedApplyService.selectById(id));
        result.put("code", 10000);
        result.put("message", "查询需求申请详情成功");
        return result;
    }

}
