package com.ff.apimngr.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ff.apimngr.dao.DynamicTableMapper;
import com.ff.apimngr.entity.DataApi;
import com.ff.apimngr.entity.ReportDatasource;
import com.ff.apimngr.service.DataApiService;
import com.ff.apimngr.service.DynamicTableService;
import com.ff.apimngr.service.ReportDatasourceService;
import com.ff.apimngr.tool.api.R;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
@Slf4j
@RestController
@RequestMapping("/api/open")
public class ApiOpenController {
     @Autowired
     private DataApiService dataApiService;
     @Autowired
     private ReportDatasourceService reportDatasourceService;
     @Autowired
     private DynamicTableService dynamicTableService;


    @GetMapping("/{apiKey}")
    @ApiOperation(value = "执行常规类接口", notes = "传入apiKey【说明：第三方接口传递apiParam(接口参数)】")
    public R executeDataApi(@PathVariable("apiKey") String apiKey, String apiParam, Integer pageNum, HttpServletRequest request) {
        log.info("executeDataApi："+apiKey);
        if (StrUtil.isBlank(apiKey)) {
            return R.fail("接口关键词不可以为空");
        }
        DataApi dataApi = dataApiService.getByApiKey(apiKey);
        if (null == dataApi) {
            return R.fail("接口不存在，请检查接口关键词");
        }
        if(dataApi.getApiType() == 2) { //第三方接口
            //获取header
            Map<String, String> headers = null;
            if(!StrUtil.isEmpty(dataApi.getApiHeaders())) {
                headers = JSON.parseObject(dataApi.getApiHeaders(), Map.class);
            }
            JSONObject queries = new JSONObject();
            String postJson = apiParam == null ? "" : apiParam;
            if(!StrUtil.isEmpty(apiParam)) {
                queries = JSON.parseObject(apiParam);
            }
            if("post".equalsIgnoreCase(dataApi.getApiMethod())) {
                if(dataApi.getApiParamType() == 0) { //JSON方式
                    return R.data(HttpUtil.createPost(dataApi.getApiUrl()).addHeaders(headers).body(postJson).execute().body());
                }else {
                    return R.data(HttpUtil.createPost(dataApi.getApiUrl()).addHeaders(headers).form(queries).execute().body());
                }
            }else {
                return R.data(HttpUtil.createGet(dataApi.getApiUrl()).addHeaders(headers).form(queries).execute().body());
            }
        }else {
            if (null == dataApi.getDatasourceId()) {
                return R.fail("接口未配置数据源");
            }
            if (1 != dataApi.getApiType()) {
                return R.fail("接口类型配置错误");
            }
//			if (null != dataApi.getLoginRequired() && 1 == dataApi.getLoginRequired()) {
//				if (null == user) {
//					return R.fail("未登录");
//				}
//			}
            String querySql = dataApi.getQuerySql();
            Map<String, String[]> maps = request.getParameterMap();
            String regex = "\\@(.*?)\\@";
            Pattern p = Pattern.compile(regex);
            Matcher m = p.matcher(querySql);
            //Map<String, Object> params = new HashMap<>();
            List<String> paramList = new ArrayList<>();
            while (m.find()) {
                String key = m.group(1);
                if (StrUtil.isNotBlank(key)) {
                    paramList.add(key);
                }
            }
            String regex2 = "\\@@(.*?)\\@@";
            Pattern p2 = Pattern.compile(regex2);
            Matcher m2 = p2.matcher(querySql);
            while (m2.find()) {
                String key = m2.group(1);
                if (StrUtil.isNotBlank(key)) {
                    paramList.add(key);
                }
            }
            ReportDatasource datasource = reportDatasourceService.getById(dataApi.getDatasourceId());
            if (null == datasource) {
                return R.fail("接口数据源配置错误");
            }
            for (String key : paramList) {
                String value = "";
                if (maps.size() > 0 && null != maps.get(key)) {
                    value = maps.get(key)[0];
                }
                if (querySql.contains("@@" + key + "@@")) {
                    if (StrUtil.isNotBlank(value)) {
                        querySql = querySql.replaceAll("@@" + key + "@@", value);
                    } else {
                        querySql = querySql.replaceAll("@@" + key + "@@", "1=1 ");
                    }
                } else if (querySql.contains("@" + key + "@")) {
                    querySql = querySql.replace("@" + key + "@", "'" + value + "'");
                }

            }
            Map<String, Object> result = new HashMap<>();
            result.put("current", pageNum);
			List<Map<String, Object>> list =new ArrayList<>();
            if (dataApi.getIsPage()== 0){
                list = dynamicTableService.selectList(datasource,querySql);
                result.put("records", list);
                result.put("total", list.size());
            }else {
                IPage<?> page = new Page<>();
                page.setSize(dataApi.getPageSize());
                if (pageNum == null){
                    pageNum=1;
                }
                page.setCurrent(pageNum);
                IPage<Map<String, Object>> selectPage = dynamicTableService.selectPage(page,datasource, querySql);
//				Integer count = queryWrapperService.selectCount(datasource, "select count(1) "+querySql.substring(querySql.lastIndexOf("from " + dataApi.getQueryTable())));
                result.put("records", selectPage != null ? selectPage.getRecords() : null);
                result.put("total", selectPage != null ? selectPage.getTotal() : 0);
            }
            return R.data(result);
        }

    }
}
