package com.ff.apimngr.controller;


import com.ff.apimngr.entity.*;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.*;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 应用管理
 */
@RestController
@RequestMapping("/api/app")
public class AppController {

    @Resource
    private AppService appService;

    @Resource
    private AppTypeService appTypeService;

    @Resource
    private UserService userService;

    @Resource
    private ApiService apiService;

    @Resource
    private AppApiService appApiService;

    @ApiOperation(value = "根据id获取应用", notes = "根据id获取应用",tags = "app管理",hidden = true)
    @GetMapping(value = "/findbyid")
    public Map<String, Object> getAppById(@ApiParam(value = "应用id", required = true) @RequestParam Integer id, HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        Integer userId = null;
        String token = request.getHeader("token");
        String account = JwtHelper.parseToken(token);
        User user = null;
        App app = appService.findAppById(id);
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "未查询到此应用");
            return result;
        }
        if (null != account && account.length() > 0) {
            user = userService.findUserByAccount(account);
            if (null == user) {
                result.put("code", 10001);
                result.put("message", "接口调用失败");
                return result;
            }
            Role role = user.getRole();
            if (null != role) {
                if (role.getRoleType() != 1) {
                    //开发者查询
                    User owner = app.getUser();
                    if (null == owner) {
                        result.put("code", 10001);
                        result.put("message", "系统错误");
                        return result;
                    }
                    if (!owner.getAccount().equals(user.getAccount())) {
                        result.put("code", 10001);
                        result.put("message", "系统错误");
                        return result;
                    }
                    userId = user.getId();
                }
            } else {
                result.put("code", 10002);
                result.put("message", "用户角色获取失败，请重新登录或者联系管理员");
                return result;
            }
        } else {
            result.put("code", 10002);
            result.put("message", "请重新登录");
            return result;
        }


        List<App> list = new ArrayList<>();
        app.setUser(user);
        list.add(app);
        result.put("data", list);
        result.put("total", list.size());

        List<AppApi> dataModel = appApiService.listAppApisByAppId(id, userId);
        List<Integer> apiIds = new ArrayList<>();
        if (null != dataModel && dataModel.size() > 0) {
            for (AppApi appApi : dataModel) {
                Api api = appApi.getApi();
                apiIds.add(api.getId());
            }
        }
        result.put("dataModel", apiIds);
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }

    @ApiOperation(value = "根据名称获取应用", notes = "根据名称获取应用",tags = "app管理",hidden = true)
    @GetMapping(value = "/findbyname")
    public Map<String, Object> findAppByName(@ApiParam(value = "应用名称", required = true) @RequestParam String name) {
        Map<String, Object> result = new HashMap<>();
        App app = appService.findAppByName(name);
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "未查询到此应用");
            return result;
        }
        result.put("data", app);
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }

    @ApiOperation(value = "创建应用", notes = "创建应用",tags = "app管理",hidden = true)
    @PostMapping(value = "/insert")
    public Map<String, Object> insertApp(@ApiParam(value = "应用名字", required = true) @RequestParam String name,
                                         @ApiParam(value = "应用类型id", required = true) @RequestParam String appTypeId,
                                         @ApiParam(value = "应用接口", required = true) @RequestParam String apis,
                                         @ApiParam(value = "到期日期", required = false) @RequestParam String expireDate,
                                         @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        if (null == token || (null != token && token.length() == 0)) {
            result.put("code", 10001);
            result.put("message", "登录失效");
            return result;
        }
        String account = JwtHelper.parseToken(token);
        if (null == account || (null != account && account.length() == 0)) {
            result.put("code", 10002);
            result.put("message", "登录失效");
            return result;
        }
        User user = userService.findUserByAccount(account);
        if (null == user) {
            result.put("code", 10003);
            result.put("message", "登录失效");
            return result;
        }
        //检查应用名+用户，是否唯一
        App app = appService.findbyUserIdAndAppName(user.getId(), name);
        if (null != app) {
            result.put("code", 10004);
            result.put("message", "同一个用户不能创建相同名称应用");
            return result;
        } else {
            app = new App();
        }

        app.setName(name);
        app.setUser(user);
        app.setAppType(appTypeService.findAppTypeById(Integer.valueOf(appTypeId)));
        Date now = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        app.setCreateTime(sdf.format(now));

        app.setSecretKey(genSecretKey());
        app.setApiKey(genApiKey());
        app.setStatus(2);//待审核
        if (null != expireDate && expireDate.length() > 0) {
            app.setExpireDate(expireDate);
        } else {
            app.setExpireDate("");
        }
        if (null == description) {
            app.setDescription("");
        } else {
            app.setDescription(description);
        }

        appService.insertApp(app);

        if (null != apis && apis.length() > 0) {
            String[] apiList = apis.split(",");
            //保存到app_api
            for (String apiId : apiList) {
                AppApi appApi = new AppApi();
                appApi.setApp(app);
                Api api = apiService.findApiById(Integer.valueOf(apiId));
                appApi.setApi(api);
                appApi.setCallLimit(api.getCallLimit());
                appApi.setQpsLimit(api.getQpsLimit());
                appApi.setStatus(0);
                appApi.setEnable(0);
                appApiService.insertAppApi(appApi);
            }
        }

        result.put("code", 10000);
        result.put("message", "创建成功");
        return result;
    }

    @ApiOperation(value = "管理员创建应用", notes = "管理员创建应用",tags = "app管理")
    @PostMapping(value = "/insertbyadmin")
    public Map<String, Object> insertAppByAdmin(@ApiParam(value = "应用名字", required = true) @RequestParam String name,
                                                @ApiParam(value = "开发者id", required = true) @RequestParam String userId,
                                                @ApiParam(value = "应用类型id", required = true) @RequestParam String appTypeId,
                                                @ApiParam(value = "应用接口", required = true) @RequestParam String apis,
                                                @ApiParam(value = "到期日期", required = false) @RequestParam String expireDate,
                                                @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description,
                                                HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();

        User user = userService.findUserById(Integer.valueOf(userId));
        if (null == user) {
            result.put("code", 10001);
            result.put("message", "开发者不存在，创建失败");
            return result;
        }
        //检查应用名+用户，是否唯一
        App app = appService.findbyUserIdAndAppName(Integer.valueOf(userId), name);
        if (null != app) {
            result.put("code", 10004);
            result.put("message", "同一个用户不能创建相同名称应用");
            return result;
        } else {
            app = new App();
        }
        app.setName(name);
        app.setUser(user);
        app.setAppType(appTypeService.findAppTypeById(Integer.valueOf(appTypeId)));
        Date now = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        app.setCreateTime(sdf.format(now));

        app.setSecretKey(genSecretKey());
        app.setApiKey(genApiKey());
        app.setStatus(1);//管理员创建，不用审核
        app.setExpireDate(expireDate);
        app.setDescription(description);

        appService.insertApp(app);

        if (null != apis && apis.length() > 0) {
            String[] apiList = apis.split(",");
            //保存到app_api
            for (String apiId : apiList) {
                AppApi appApi = new AppApi();
                appApi.setApp(app);
                Api api = apiService.findApiById(Integer.valueOf(apiId));
                appApi.setApi(api);
                appApi.setCallLimit(api.getCallLimit());
                appApi.setQpsLimit(api.getQpsLimit());
                appApi.setStatus(1);
                appApiService.insertAppApi(appApi);
            }
        }

        result.put("code", 10000);
        result.put("message", "创建成功");
        return result;
    }


    @ApiOperation(value = "更新应用", notes = "更新应用",tags = "app管理")
    @PostMapping(value = "/update")
    public Map<String, Object> updateApp(@ApiParam(value = "应用id", required = true) @RequestParam String id,
                                         @ApiParam(value = "应用名字", required = true) @RequestParam String name,
                                         @ApiParam(value = "应用类型id", required = true) @RequestParam String appTypeId,
                                         //@ApiParam(value = "应用接口", required = false) @RequestParam String apis,
                                         @ApiParam(value = "到期日期", required = false) @RequestParam String expireDate,
                                         @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description) {
        Map<String, Object> result = new HashMap<>();
        App app = appService.findAppById(Integer.valueOf(id));
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "应用不存在");
            return result;
        }
        app.setName(name);

        app.setDescription(description);
        app.setExpireDate(expireDate);
        app.setAppType(appTypeService.findAppTypeById(Integer.valueOf(appTypeId)));

        /*appApiService.deleteAppApisByAppId(Integer.valueOf(id));
        if (null != apis && apis.length() > 0) {
            String[] apiList = apis.split(",");
            //保存到app_api
            for (String apiId : apiList) {
                AppApi appApi = new AppApi();
                appApi.setApp(app);
                Api api = apiService.findApiById(Integer.valueOf(apiId));
                appApi.setApi(api);
                appApi.setCallLimit(api.getCallLimit());
                appApi.setQpsLimit(api.getQpsLimit());
                appApi.setStatus(1);
                appApiService.insertAppApi(appApi);
            }
        }*/

        boolean rst = appService.updateApp(app);
        result.put("code", 10000);
        result.put("message", "更新成功");
        return result;
    }


    @ApiOperation(value = "根据id删除应用", notes = "根据id删除应用",tags = "app管理")
    @PostMapping(value = "/delete")
    public Map<String, Object> deleteApp(@ApiParam(value = "应用id", required = true) @RequestParam String id,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = "";
        account = JwtHelper.parseToken(token);
        User user = userService.findUserByAccount(account);
        if (null == user) {
            result.put("code", 10001);
            result.put("message", "删除失败，用户信息错误");
            return result;
        }
        Role role = user.getRole();
        App app = appService.findAppById(Integer.valueOf(id));
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "删除失败，应用不存在");
            return result;
        }
        if (role.getRoleType() != 1) {
            //非管理员，判断是否是自己的app，否则不允许删除
            User owner = app.getUser();
            if (null == owner) {
                result.put("code", 10001);
                result.put("message", "删除失败，系统错误");
                return result;
            }
            if (!owner.getAccount().equals(user.getAccount())) {
                result.put("code", 10001);
                result.put("message", "删除失败，系统错误");
                return result;
            }
        }

        boolean rst = appService.deleteAppById(Integer.valueOf(id));
        if (rst) {
            result.put("code", 10000);
            result.put("message", "删除成功");
            //删除完应用，将应用的接口权限列表删除
            appApiService.deleteAppApisByAppId(Integer.valueOf(id));
        } else {
            result.put("code", 10002);
            result.put("message", "删除失败");
        }

        return result;
    }

    @ApiOperation(value = "根据id审核应用", notes = "根据id审核应用",tags = "app管理",hidden = true)
    @PostMapping(value = "/approve")
    public Map<String, Object> approveApp(@ApiParam(value = "应用id", required = true) @RequestParam String id,
                                          @ApiParam(value = "审批结果", required = true) @RequestParam Integer status) {
        Map<String, Object> result = new HashMap<>();
        App app = appService.findAppById(Integer.valueOf(id));
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "审批失败，应用不存在");
            return result;
        }
        app.setStatus(status);
        appService.updateApp(app);
        result.put("code", 10000);
        result.put("message", "审核成功");

        return result;
    }

    @ApiOperation(value = "根据id停用/启动应用", notes = "根据id停用/启动应用",tags = "app管理")
    @PostMapping(value = "/disable")
    public Map<String, Object> disableApi(@ApiParam(value = "接口id", required = true) @RequestParam String id,
                                          @ApiParam(value = "停用/启用应用", required = true) @RequestParam Integer status) {
        Map<String, Object> result = new HashMap<>();
        App app = appService.findAppById(Integer.valueOf(id));
        if (null == app) {
            result.put("code", 10001);
            result.put("message", "更新应用失败，应用不存在");
            return result;
        }
        app.setStatus(status);
        appService.updateApp(app);
        result.put("code", 10000);
        result.put("message", "停用启用应用成功");

        return result;
    }


    @ApiOperation(value = "获取应用列表", notes = "获取应用列表",tags = "app管理")
    @GetMapping(value = "/list")
    public Map<String, Object> searchApps(@ApiParam(value = "搜索关键字", required = false) @RequestParam(required = false) String tag,
                                          @ApiParam(value = "应用状态", required = false) @RequestParam(required = false) Integer status,
                                          @ApiParam(value = "用户id", required = false) @RequestParam(required = false) Integer id,
                                          @ApiParam(value = "预警状态 1=预警应用 2=异常应用", required = false) @RequestParam(required = false) Integer alarmStatus,
                                          @ApiParam(value = "到期日期", required = false) @RequestParam(required = false) String expireDate,
                                          @ApiParam(value = "appTypeId", required = false) @RequestParam(required = false) Integer appTypeId,
                                          HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = JwtHelper.parseToken(token);
        if (null == account || (null != account && account.length() == 0)) {
            result.put("code", 10002);
            result.put("message", "登陆失效，请重新登录");
            return result;
        }
        User user = userService.findUserByAccount(account);
        if (null == user) {
            result.put("code", 10003);
            result.put("message", "登陆失效，请重新登录");
            return result;
        }
        Integer userId = null;
        Role role = user.getRole();
        if (null == role) {
            result.put("code", 10004);
            result.put("message", "登陆失效，请重新登录");
            return result;
        }
        Integer roleType = role.getRoleType();
        if (roleType == 1) {
            //管理员
            if (null != id) {
                userId = id;
            }
        } else {
            userId = user.getId();
        }

        List<App> list = appService.listApps(tag, status, userId, alarmStatus, expireDate,appTypeId);
        if (null != list) {
            for (App app : list) {
                if(app.getStatus() == 1){//正常状态下 校验是否还有未审核通过的接口
                    Integer count = appApiService.getListByAppId(app.getId());
                    if(count > 0){
                        app.setStatus(99);
                    }
                }
            }
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询失败");
        }
        return result;
    }


    @ApiOperation(value = "获取已应用总数", notes = "获取已应用总数",tags = "app管理",hidden = true)
    @GetMapping(value = "/listcount")
    public Map<String, Object> searchAppsCount(@ApiParam(value = "用户id", required = false) @RequestParam(required = false) String selectedId,
                                               HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = JwtHelper.parseToken(token);
        Integer userId = null;
        if (null != account && account.length() > 0) {
            User user = userService.findUserByAccount(account);
            if (null == user) {
                result.put("code", 10001);
                result.put("message", "接口调用失败");
                return result;
            }
            Role role = user.getRole();
            if (null != role) {
                if (role.getRoleType() != 1) {
                    //开发者查询
                    userId = user.getId();
                } else {
                    //管理员默认查询所有数据
                    if (null != selectedId && selectedId.length() > 0) {
                        //管理员选择某个用户统计
                        try {
                            userId = Integer.parseInt(selectedId);
                        } catch (Exception e) {
                            e.printStackTrace();
                            result.put("code", 10004);
                            result.put("message", "用户id错误，请联系管理员");
                            return result;
                        }
                    }
                }
            } else {
                result.put("code", 10002);
                result.put("message", "用户角色获取失败，请重新登录或者联系管理员");
                return result;
            }
        }
        if (null != selectedId && selectedId.length() > 0) {
            userId = Integer.parseInt(selectedId);
        }
        Integer count = appService.listAppsCount(userId);
        Integer recentCount = appService.listRecentAppCount(userId);
        if (null != count) {
            result.put("total", count);
            result.put("recentCount", recentCount);
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询失败");
        }

        return result;
    }

    private String genApiKey() {
        String uuid = UUID.randomUUID().toString();
        uuid = uuid.replace("-", "");
        return uuid.substring(0, 24);
    }

    private String genSecretKey() {
        String uuid = UUID.randomUUID().toString();
        uuid = uuid.replace("-", "");
        return uuid.substring(0, 32);
    }

    @ApiOperation(value = "获取应用数量统计", notes = "获取应用数量统计",tags = "app管理",hidden = true)
    @GetMapping(value = "/appcount")
    public Map<String, Object> searchDevelopersReport() throws ParseException {
        Map<String, Object> result = new HashMap<>();

        result.put("allAppNum", appService.getAllAppNum());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String beginDate = format.format(new Date()) + " 00:00:00";
        Date beginDay = sdf.parse(beginDate);
        result.put("todayActiveAppNum", appService.getTodayActiveAppNum(beginDay.getTime()));


        Calendar cale = Calendar.getInstance();
        // 获取前月的第一天
        cale.add(Calendar.MONTH, 0);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        Date monthDay = sdf.parse(format.format(cale.getTime()) + " 00:00:00");
        result.put("monthAcitveAppNum", appService.getTodayActiveAppNum(monthDay.getTime()));
        result.put("alaramAppNum", appService.getAlarmAppNum());
        result.put("abnormalAppNum", appService.getAbnormalAppNum());
        result.put("illegalAppNum", appService.getIllegalAppNum());
        result.put("message", "查询完成");
        result.put("code", 10000);


        return result;
    }

//    @ApiOperation(value = "获取应用活跃排行榜", notes = "获取应用活跃排行榜",tags = "app管理",hidden = true)
//    @GetMapping(value = "/topapps")
//    public Map<String, Object> searchTopDevelopers(@ApiParam(value = "开始日期", required = false) @RequestParam(required = false) String beginDate,
//                                                   @ApiParam(value = "结束日期", required = false) @RequestParam(required = false) String endDate) throws ParseException {
//        Map<String, Object> result = new HashMap<>();
////        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
////        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
////        if (null == beginDate) {
////            beginDate = format.format(new Date());
////        }
////        if (null == endDate) {
////            endDate = format.format(new Date());
////        }
////        Date beginDay = sdf.parse(beginDate + " 00:00:00");
////        Date endDay = sdf.parse(endDate + " 23:59:59");
////        Long beginTime = beginDay.getTime();
////        Long endTime = endDay.getTime();
//        List<TopApp> list = appService.getTopTenApps(beginDate, endDate);
//        if (null != list && list.size() > 0) {
//            int i = 1;
//            List<TopApp> topApps = new ArrayList<>();
//            for (TopApp topApp : list) {
//                topApp.setId(i);
//                i++;
//                topApps.add(topApp);
//            }
//            result.put("data", topApps);
//            result.put("total", list.size());
//            result.put("code", 10000);
//            result.put("message", "查询成功");
//        } else {
//            result.put("code", 10001);
//            result.put("message", "没有查询到结果");
//        }
//        return result;
//    }

}
