package com.ff.apimngr.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ff.apimngr.entity.*;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.*;
import com.ff.apimngr.support.Condition;
import com.ff.apimngr.support.Query;
import com.ff.apimngr.tool.api.R;
import com.ff.apimngr.tool.utils.Func;
import com.ff.apimngr.util.CubeUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 数据接口 控制器
 *
 * @author BladeX
 * @since 2021-04-17
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("api/report/dataapi")
@Api(value = "数据接口", tags = "数据接口接口 DataApiController")
public class DataApiController  {
    @Autowired
	private DataApiService dataApiService;
    @Autowired
    private ApiService apiService;

    @Autowired
    private ApiTypeService apiTypeService;

    @Autowired
    private UserService userService;

    @Autowired
    private ApiVersionService apiVersionService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperation(value = "详情 detail", notes = "传入dataApi")
	public R<DataApi> detail(DataApi dataApi) {
		DataApi detail = dataApiService.getOne(Condition.getQueryWrapper(dataApi));
		return R.data(detail);
	}

	/**
	 * 分页 数据接口
	 */
	@GetMapping("/list")
	@ApiOperation(value = "列表 list", notes = "传入dataApi")
	public R<List<DataApi>> list(DataApi dataApi, String apiTypes) {
		QueryWrapper<DataApi> queryWrapper = Condition.getQueryWrapper(dataApi);
		if (StrUtil.isNotBlank(apiTypes)) {
			String[] apiTypeArr = apiTypes.split(",");
			queryWrapper.lambda().in(DataApi::getApiType, Stream.of(apiTypeArr).map(Integer::valueOf).collect(Collectors.toList()));
		}
		if (StrUtil.isNotBlank(dataApi.getApiName())) {
			queryWrapper.lambda().like(DataApi::getApiName, dataApi.getApiName());
		}
		List<DataApi> list = dataApiService.list(queryWrapper);
		return R.data(list);
	}

	/**
	 * 自定义分页 数据接口
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页 page", notes = "传入dataApi")
	public R<IPage<DataApi>> page(DataApi dataApi, Query query) {
//		dataApi.setCreateUserId(getUser().getUserId());
		QueryWrapper<DataApi> queryWrapper = Condition.getQueryWrapper(dataApi);
		return R.data(dataApiService.page(Condition.getPage(query), queryWrapper));
	}

	/**
	 * 新增或修改 数据接口
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "新增或修改", notes = "传入dataApi")
	public R<DataApi> submit(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		DataApiDTO dataApiDTO = JSON.parseObject(decoderStr, DataApiDTO.class);;
		DataApi dataApi = dataApiDTO;
		if (null == dataApi.getApiType()) {
			return R.fail("API类型不能为空");
		}
		if (StrUtil.isBlank(dataApi.getApiKey())) {
			return R.fail("api关键词不可以为空");
		}
		if (2 != dataApi.getApiType() && null == dataApi.getDatasourceId()) {
			return R.fail("api数据源不可以为空");
		}
		if (1 == dataApi.getApiType() && StrUtil.isBlank(dataApi.getQuerySql())) {
			return R.fail("常规接口查询sql不可以为空");
		}
		if (1 == dataApi.getApiType() && checkSqlKeywords(dataApi.getQuerySql())) {
			return R.fail("查询sql包含非法关键字");
		}
		if (3 == dataApi.getApiType() && StrUtil.isBlank(dataApi.getQueryParam())) {
			return R.fail("业务参数不能为空");
		}
		if (1 == dataApi.getApiType()) {
			dataApi.setApiUrl("/report/dataapi/" + dataApi.getApiKey());
		}else if(3 == dataApi.getApiType()) {
			dataApi.setApiUrl("/data-api/" + dataApi.getApiKey());
		}  else if (0 == dataApi.getApiType()) {
			dataApi.setApiUrl("/report/reportApi/" + dataApi.getApiKey());
		}
		dataApiService.saveOrUpdate(dataApi);
		return R.data(dataApi);
	}


	/**
	 * 删除 数据接口
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(dataApiService.removeByIds(Func.toLongList(ids)));
	}



	@PostMapping("/check")
	@ApiOperation(value = "检测ApiKey是否存在接口", notes = "传入apiKey")
	public R<Boolean> checkApiKey(@RequestParam String apiKey){
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("api_key", apiKey);
		DataApi dataApi = dataApiService.getOne(queryWrapper);
		if (null == dataApi) {
			return R.data(false, "接口不存在");
		}else {
			return R.data(true, "接口存在");
		}
	}
	@PostMapping("/release")
	@ApiOperation(value = "发布api到数据门户", notes = "发布api到数据门户")
	public R release(
					 @RequestBody ApiReleaseDTO apiReleaseDTO,
					 HttpServletRequest request){
//		Map<String, Object> paramMap =new HashMap<>();
//		paramMap.put("name",name);//接口名字
//		paramMap.put("url",url);//请求地址
//		paramMap.put("version",version);//接口版本
//		paramMap.put("description",description);//接口备注
//		paramMap.put("callLimit",callLimit);//调用量限制
//		paramMap.put("qpsLimit",qpsLimit);//QPS限制
//		paramMap.put("status",status);//状态  1=正常 0=不正常 2=测试（未发布） 3=VIP
//		paramMap.put("apiTypeId",apiTypeId);//接口类型id
//		paramMap.put("extJson",extJson);
        com.ff.apimngr.entity.Api api = apiService.findApiByName(apiReleaseDTO.getName());
        if (null != api) {
            return R.fail("接口名已存在");
        }
        api = new com.ff.apimngr.entity.Api();
        api.setName(apiReleaseDTO.getName());
        api.setUrl(apiReleaseDTO.getUrl());
        api.setCallLimit(apiReleaseDTO.getCallLimit());
        api.setQpsLimit(apiReleaseDTO.getQpsLimit());
        api.setExtJson(apiReleaseDTO.getExtJson());
        api.setApiType(apiTypeService.findApiTypeById(Integer.valueOf(apiReleaseDTO.getApiTypeId())));
        api.setCreateBy(getAccount(request));
        api.setApplyStatus(2);
        api.setStatus(apiReleaseDTO.getStatus());// 1=正常 0=不正常 2=测试（未发布） 3=VIP
        if (null == apiReleaseDTO.getDescription()) {
            api.setDescription("");
        } else {
            api.setDescription(apiReleaseDTO.getDescription());
        }

        if (null == apiReleaseDTO.getVersion()) {
            api.setVersion("");
        } else {
            api.setVersion(apiReleaseDTO.getVersion());
        }
        if (null == apiReleaseDTO.getDocUrl()) {
            api.setDocUrl("");
        } else {
            api.setDocUrl(apiReleaseDTO.getDocUrl());
        }
        boolean ret = apiService.insertApi(api);
        if (ret) {
            // 插入版本记录
            ApiVersion apiVersion = new ApiVersion();
            apiVersion.setApiId(api.getId());
            apiVersion.setCreateBy(getAccount(request));
            apiVersion.setOldVersion(apiReleaseDTO.getVersion());
            apiVersion.setRemark(apiReleaseDTO.getDescription());
            apiVersionService.save(apiVersion);
        }
        return R.success("接口发布成功!");
    }

	Boolean checkSqlKeywords(String sql) {
		return CubeUtils.checkSqlInject(sql);
	}

    String getAccount(HttpServletRequest request) {
        String token = request.getHeader("token");
        String account = "";
        if (token != null && token.length() > 0) {
            account = JwtHelper.parseToken(token);

            User user = userService.findUserByAccount(account);
            if (null != user) {
                return user.getAccount();
            }
        }
        return "";
    }
}
