package com.ff.apimngr.controller;

import com.ff.apimngr.dao.ApiLogMapper;
import com.ff.apimngr.dao.AppMapper;
import com.ff.apimngr.entity.*;
import com.ff.apimngr.service.ApiService;
import com.ff.apimngr.service.ApiTypeService;
import com.ff.apimngr.service.AppService;
import com.ff.apimngr.util.RedisUtil;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/homepage")
public class HomePageController {

    @Resource
    private ApiService apiService;

    @Resource
    private ApiTypeService apiTypeService;

    @Autowired
    private AppService appService;
    @Autowired
    private AppMapper appMapper;

    @Autowired
    private ApiLogMapper apiLogMapper;

    @Resource
    private RedisUtil redisUtil;


    @ApiOperation(value = "数据统计", notes = "数据统计", tags = "接口管理")
    @GetMapping(value = "/overview")
    public Map<String, Object> overview(
             @RequestParam(required = false) String apiId,
             @RequestParam(required = false) String appId,
             @RequestParam(required = false) String userId,
             @RequestParam(required = false) Integer type

    ) {
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
             apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
             appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
             userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());

        }
        Map<String, Object> result = new HashMap<>();
        //数据对象
        Map<String, Object> dataMap = new HashMap<>();
        //应用数
        dataMap.put("appNum", appService.appCount(appIds,userIds,type));
        //申请API数
        dataMap.put("apiApplyNum", apiService.getApplyNum(userIds));
        CountResult apiCallNum = apiLogMapper.getApiCallNum(appIds,apiIds,userIds,type);
        dataMap.put("total", apiCallNum.getTotal());
        dataMap.put("successCount", apiCallNum.getSuccessCount());
        dataMap.put("failCount", apiCallNum.getFailCount());
        result.put("data", dataMap);
        result.put("code", 10000);
        result.put("message", "获取成功");
        // 增加浏览数量
        return result;
    }


    @ApiOperation(value = "应用使用情况", notes = "应用使用情况", tags = "应用使用情况")
    @GetMapping(value = "/appByDate")
    public Map<String, Object> appByDate(   @RequestParam(required = false) String apiId,
                                            @RequestParam(required = false) String appId,
                                            @RequestParam(required = false) String userId,
                                            @RequestParam(required = false) String type) {
        Map<String, Object> result = new HashMap<>();
        System.out.println("appByDate  apiId = " + apiId + " appId = " + appId + " userId = " + userId + " type = " + type);
        OverviewVO overviewVO =new OverviewVO();
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
            apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
            appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
            userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());

        }
        overviewVO.setApiIds(apiIds);
        overviewVO.setAppIds(appIds);
        overviewVO.setUserIds(userIds);
        List<AppByDateVO> appByDateVOS = appMapper.appUse(apiIds, appIds, userIds, type);
        result.put("data", appMapper.appUse(apiIds,appIds,userIds,type));
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }
    @ApiOperation(value = "实时调用情况", notes = "实时调用情况", tags = "实时调用情况")
    @GetMapping(value = "/realTimeCall")
    public Map<String, Object> realTimeCall(@RequestParam(required = false) String apiId,
                                            @RequestParam(required = false) String appId,
                                            @RequestParam(required = false) String userId,
                                            @RequestParam(required = true) String type) {
        Map<String, Object> result = new HashMap<>();
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
            apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
            appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
            userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if ("0".equals(type)){
            result.put("data",apiLogMapper.getRealTimeCallToday(apiIds,appIds,userIds));
        }else if ("1".equals(type)){
            result.put("data",apiLogMapper.getRealTimeCallWeek(apiIds,appIds,userIds));
        }else if ("2".equals(type)){
            result.put("data",apiLogMapper.getRealTimeCallMonth(apiIds,appIds,userIds));
        }
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }
    @ApiOperation(value = "获取应用活跃排行榜", notes = "获取应用活跃排行榜",tags = "app管理",hidden = true)
    @GetMapping(value = "/topapps")
    public Map<String, Object> topapps(@RequestParam(required = false) String apiId,
                                                   @RequestParam(required = false) String appId,
                                                   @RequestParam(required = false) String userId,
                                                   @RequestParam(required = true) Integer type)  {
        Map<String, Object> result = new HashMap<>();
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
            apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
            appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
            userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        List<TopApp> list =appMapper.getTopTenApps(appIds, apiIds,userIds,type);
        result.put("data",  list);
        result.put("total", list.size());
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }


    @ApiOperation(value = "获取能力调用排行榜", notes = "获取能力调用排行榜", hidden = true)
    @GetMapping(value = "/topapis")
    public Map<String, Object> topapis(
            @RequestParam(required = false) String apiId,
            @RequestParam(required = false) String appId,
            @RequestParam(required = false) String userId,
            @RequestParam(required = true) Integer type) {
        Map<String, Object> result = new HashMap<>();
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
            apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
            appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
            userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        List<TopApi> list = apiService.getTopTenApis(appIds, apiIds,userIds,type);
        result.put("data", list);
        result.put("total", list.size());
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }
    @ApiOperation(value = "省份分布", notes = "省份分布", hidden = true)
    @GetMapping(value = "/region")
    public Map<String, Object> region(
            @RequestParam(required = false) String apiId,
            @RequestParam(required = false) String appId,
            @RequestParam(required = false) String userId,
            @RequestParam(required = true) String type) {
        Map<String, Object> result = new HashMap<>();
        List<Integer> apiIds=new ArrayList<>();
        List<Integer> appIds=new ArrayList<>();
        List<Integer> userIds=new ArrayList<>();
        if (!StringUtils.isEmpty(apiId)){
            apiIds = Arrays.asList(apiId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(appId)){
            appIds = Arrays.asList(appId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        if (!StringUtils.isEmpty(userId)){
            userIds = Arrays.asList(userId.split(",")).stream().map(Integer::parseInt).collect(Collectors.toList());
        }
        result.put("data", apiLogMapper.getRegion(appIds, apiIds,userIds,type));
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }

}
