/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package com.ff.apimngr.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.PageUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ff.apimngr.dto.oracle.ReportViewDTO;
import com.ff.apimngr.dto.response.TableCategoryDTO;
import com.ff.apimngr.entity.ReportDatasource;
import com.ff.apimngr.entity.TableCategory;
import com.ff.apimngr.entity.TableCategoryRelation;
import com.ff.apimngr.service.DynamicTableService;
import com.ff.apimngr.service.ReportDatasourceService;
import com.ff.apimngr.service.TableCategoryRelationService;
import com.ff.apimngr.service.TableCategoryService;
import com.ff.apimngr.support.Condition;
import com.ff.apimngr.support.Query;
import com.ff.apimngr.tool.api.R;
import com.ff.apimngr.tool.exception.ServiceException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.model.DatasourceDto;
import org.springblade.datasource.model.TableColumnDto;
import org.springblade.datasource.model.TableViewDto;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.sql.SQLException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 产品报表基础信息 控制器
 *
 * @author BladeX
 * @since 2021-03-10
 */
@RestController
@RequestMapping("api/report/baseInfo")
@Api(value = "数据库表基础信息", tags = "数据库表基础信息 ReportBaseController")
public class ReportBaseController  {

//    @Resource
//    private ReportBaseService reportBaseService;

    @Resource
    private ReportDatasourceService reportDatasourceService;

    @Autowired
    private DialectContext dialectContext;


    @Autowired
    private TableCategoryRelationService tableCategoryRelationService;

    @Autowired
    private TableCategoryService tableCategoryService;

    @Autowired
    private DynamicTableService dynamicTableService;

    /**
     * 获取报表视图
     */
//    @GetMapping("/listViews")
//    @ApiOperation(value = "获取报表视图", notes = "")
//    @Deprecated
//    public R<List<ReportViewDTO>> list() {
//        List<ReportViewDTO> list = reportBaseService.selectReportViewList();
//        return R.data(list);
//    }

    /**
     * 获取报表视图
     */
    @GetMapping("/listViewsByDataSource")
    @ApiOperation(value = "根据数据源获取报表视图", notes = "")
//	@Cacheable(cacheNames = "baseInfo", key = "#datasourceId")
    public R<IPage<TableCategoryDTO>> list(HttpServletRequest request,
                                           @ApiParam("数据源Id") @RequestParam Integer datasourceId,
                                           @ApiParam("分类id") @RequestParam(required = false) Integer classifyId,
                                           @ApiParam("表名") @RequestParam(required = false) String tableName,
                                           @ApiParam("分页参数") Query query) throws ClassNotFoundException, SQLException {
        return R.data(getTableView(datasourceId, classifyId, tableName, query));
    }


    /**
     * 获取报表视图的字段信息
     */
    @GetMapping("/listColumnsByDatasource")
    @ApiOperation(value = "根据数据源获取报表视图的字段信息", notes = "")
    public R<List<TableColumnDto>> listColumnsByDatasource(ReportViewDTO reportViewDTO, HttpServletRequest request) throws ClassNotFoundException, SQLException {
        String viewName = reportViewDTO.getTableViewName();
        if (StrUtil.isAllBlank(viewName)) {
            return R.fail("视图名称不可以为空");
        }
        ReportDatasource datasource = reportDatasourceService.getById(reportViewDTO.getDatasourceId());
        if (null == datasource) {
            return R.fail("数据源不存在");
        }
        DatasourceDto ds =new DatasourceDto();
        BeanUtils.copyProperties(datasource,ds);
        List<TableColumnDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableColumnsByTableViewName(ds, viewName);
        return R.data(list);
    }


    @Deprecated
    @GetMapping("removeBaseInfoCache")
    @ApiOperation(value = "清除baseInfo缓存", notes = "")
    public R removeBaseInfoCache(Integer datasourceId) {
        return R.success("清除缓存完成");
    }

    /**
     * <p>查询表和分类数据</p>
     * @author wookvn
     * @datetime 2024-12-12 18:08:58
     * @param datasourceId: 数据源ID
     * @param classifyId: 分类ID
     * @param tableName: 表名称
     * @param query: 分页查询参数
     * @return com.baomidou.mybatisplus.core.metadata.IPage<org.springblade.report.dto.response.TableCategoryDTO>
     **/
    public IPage<TableCategoryDTO> getTableView(Integer datasourceId, Integer classifyId, String tableName, Query query) {
        IPage<TableCategoryDTO> page = Condition.getPage(query);

        ReportDatasource datasource = reportDatasourceService.getById(datasourceId);
        if (null == datasource) {
            throw new ServiceException("数据源不存在");
        }
        DatasourceDto ds =new DatasourceDto();
        BeanUtils.copyProperties(datasource,ds);
        //通过数据源查询表结构
        List<TableViewDto> list = dialectContext.getDialect(datasource.getDsTypeSyl()).listTableViewsByDatasource(ds, tableName);

        //结果集为空
        if (CollectionUtil.isEmpty(list)) {
            return page;
        }

        //排序
        list.sort(Comparator.comparing(TableViewDto::getTableViewName));

        //查询某个分类，某个数据源的表与分类的关系列表
        LambdaQueryWrapper<TableCategoryRelation> queryWrapper = Wrappers.<TableCategoryRelation>lambdaQuery()
                .eq(TableCategoryRelation::getDatasourceId, datasourceId)
                .eq(classifyId != null && classifyId >= 1, TableCategoryRelation::getCategoryId, classifyId)
                .orderByDesc(TableCategoryRelation::getId);
        List<TableCategoryRelation> categoryRelationList = Optional.<List<TableCategoryRelation>>ofNullable(tableCategoryRelationService.list(queryWrapper)).orElse(new ArrayList<>());

        //返回的数据结果List
        List<TableCategoryDTO> finDataList = new ArrayList<>();

        //设置hutool的起始页码为1，默认是0
        PageUtil.setFirstPageNo(1);
        //查询全部：填充分类  以List<TableViewDto>为主
        if (classifyId == null || classifyId < 1) {
            //设置总数
            page.setTotal(list.size());
            // 对查询结果进行逻辑分页
            List<TableViewDto> resultList = ListUtil.page(query.getCurrent(), query.getSize(), list);
            Map<String, TableCategoryRelation> categoryRelationMap = categoryRelationList.stream().collect(Collectors.toMap(TableCategoryRelation::getTableViewName, t -> t, (t1, t2) -> t1));
            //开始构造数据，并设置分类
            for (TableViewDto tableViewDto : resultList) {
                TableCategoryDTO tableCategoryDTO = BeanUtil.toBean(tableViewDto, TableCategoryDTO.class);
                TableCategoryRelation tableCategoryRelation = categoryRelationMap.get(tableViewDto.getTableViewName());
                if(tableCategoryRelation != null) {
                    tableCategoryDTO.setCategoryId(tableCategoryRelation.getCategoryId());
                    tableCategoryDTO.setTableComments(StrUtil.isNotBlank(tableCategoryRelation.getTableComments()) ? tableCategoryRelation.getTableComments() : tableCategoryDTO.getTableComments());
                }
                finDataList.add(tableCategoryDTO);
            }
        }else { //带有分类的查询，以List<TableCategoryRelation>为主, 需要去除通过sql工具删除的表
            Map<String, TableViewDto> tableViewDtoMap = list.stream().collect(Collectors.toMap(TableViewDto::getTableViewName, t -> t, (t1, t2) -> t1));
            categoryRelationList.removeIf(item -> !tableViewDtoMap.containsKey(item.getTableViewName()));
            //设置总数
            page.setTotal(categoryRelationList.size());
            // 对查询结果进行逻辑分页
            List<TableCategoryRelation> resultList = ListUtil.page(query.getCurrent(), query.getSize(), categoryRelationList);
            finDataList = BeanUtil.copyToList(resultList, TableCategoryDTO.class);
        }

        //批量设置分类名称
        if(CollectionUtil.isNotEmpty(finDataList)) {
            Set<Integer> categoryIdList = finDataList.stream().map(TableCategoryDTO::getCategoryId).filter(Objects::nonNull).collect(Collectors.toSet());
            if(CollectionUtil.isNotEmpty(categoryIdList)) {
                Map<Integer, String> categoryNameMap = Optional.ofNullable(tableCategoryService.listByIds(categoryIdList)).orElse(new ArrayList<>()).stream().collect(Collectors.toMap(TableCategory::getId, TableCategory::getCategoryName));
                finDataList.forEach(item -> item.setCategoryName(categoryNameMap.get(item.getCategoryId())));
            }
        }

        //设置数据列表
        page.setRecords(finDataList);
        return page;
    }

    @GetMapping("/data-browser")
    @ApiOperation(value = "浏览数据", notes = "支持自定义分页")
    public R<IPage<Map<String, Object>>> dataBrowser(@RequestParam Integer datasourceId,
                                                     @RequestParam String tableName,
                                                     Query query) {
        String sqlStatement = "select * from " + tableName;
        ReportDatasource reportDatasource = reportDatasourceService.getById(datasourceId);
        if (reportDatasource == null) {
            return R.fail("数据源不存在");
        }

        return R.data(dynamicTableService.selectPage(new Page<>(query.getCurrent(), query.getSize()),
                reportDatasource, sqlStatement));
    }
}
