package com.ff.apimngr.controller;


import com.ff.apimngr.entity.SystemLog;
import com.ff.apimngr.service.SystemLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 系统日志管理
 */
@Api(hidden = true)
@RestController
@RequestMapping("/api/systemlog")
public class SystemLogController {

    @Resource
    private SystemLogService systemLogService;

    @ApiOperation(value = "根据id获取系统日志", notes = "根据id获取系统日志")
    @GetMapping(value = "/findbyid")
    public Map<String, Object> getSystemLogById(@ApiParam(value = "系统日志id", required = true) @RequestParam Integer id) {
        Map<String, Object> result = new HashMap<>();
        SystemLog systemLog = systemLogService.findSystemLogById(id);
        if (null == systemLog) {
            result.put("code", 10001);
            result.put("message", "未查询到此系统日志");
            return result;
        }
        result.put("data", systemLog);
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }


    @ApiOperation(value = "根据id删除系统日志", notes = "根据id删除系统日志")
    @PostMapping(value = "/delete")
    public Map<String, Object> deleteSystemLog(@ApiParam(value = "系统日志id", required = true) @RequestParam String id) {
        Map<String, Object> result = new HashMap<>();
        SystemLog systemLog = systemLogService.findSystemLogById(Integer.valueOf(id));
        if (null == systemLog) {
            result.put("code", 10001);
            result.put("message", "删除失败，系统日志不存在");
            return result;
        }
        boolean rst = systemLogService.deleteSystemLogById(Integer.valueOf(id));
        if (rst) {
            result.put("code", 10000);
            result.put("message", "删除成功");
        } else {
            result.put("code", 10002);
            result.put("message", "删除失败");
        }

        return result;
    }

    @ApiOperation(value = "获取系统日志列表", notes = "获取系统日志列表")
    @GetMapping(value = "/list")
    public Map<String, Object> searchLogs(@ApiParam(value = "搜索关键字", required = false) @RequestParam(required = false) String tag,
                                          @ApiParam(value = "开始时间", required = false) @RequestParam(required = false) String beginTime,
                                          @ApiParam(value = "结束时间", required = false) @RequestParam(required = false) String endTime) throws ParseException {
        Map<String, Object> result = new HashMap<>();
        List<SystemLog> list = systemLogService.listSystemLogs(tag,beginTime,endTime);
        if (null != list) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询成功");
        }

        return result;
    }
}
