package com.ff.apimngr.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ff.apimngr.service.DynamicTableService;
import com.ff.apimngr.support.Query;
import com.ff.apimngr.dto.response.TableCategoryDTO;
import com.ff.apimngr.entity.*;
import com.ff.apimngr.service.ReportDatasourceService;
import com.ff.apimngr.service.TableCategoryRelationService;
import com.ff.apimngr.service.TableCategoryService;
import com.ff.apimngr.support.Condition;
import com.ff.apimngr.tool.api.R;
import com.ff.apimngr.tool.exception.ServiceException;
import com.ff.apimngr.tool.utils.Func;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 表分类 控制器
 *
 * @author BladeX
 * @since 2021-05-06
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/tablecategory")
@Api(value = "表分类", tags = "表分类接口 TableCategoryController")
public class TableCategoryController {
	@Autowired
	private TableCategoryService tableCategoryService;
	@Autowired
	private TableCategoryRelationService tableCategoryRelationService;
	@Autowired
	private ReportDatasourceService reportDatasourceService;
	@Autowired
	private DynamicTableService dynamicTableService;


	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperation(value = "详情 detail", notes = "传入tableCategory")
	public R<TableCategory> detail(TableCategory tableCategory) {
		TableCategory detail = tableCategoryService.getOne(Condition.getQueryWrapper(tableCategory));
		return R.data(detail);
	}

	/**
	 * 分页 表分类
	 */
	@GetMapping("/list")
	@ApiOperation(value = "列表 list", notes = "传入tableCategory")
	public R<List<TableCategory>> list(TableCategory tableCategory) {
		List<TableCategory> list = tableCategoryService.list(Condition.getQueryWrapper(tableCategory));
		return R.data(list);
	}

	/**
	 * 自定义分页 表分类
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页 page", notes = "传入tableCategory")
	public R<IPage<TableCategoryVO>> page(TableCategoryVO tableCategory, Query query) {
		IPage<TableCategoryVO> pages = tableCategoryService.selectTableCategoryPage(Condition.getPage(query), tableCategory);
		return R.data(pages);
	}

	/**
	 * 新增 表分类
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增 save", notes = "传入tableCategory")
	public R save(@Valid @RequestBody TableCategory tableCategory) {
		return R.status(tableCategoryService.save(tableCategory));
	}

	/**
	 * 修改 表分类
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改 update", notes = "传入tableCategory")
	public R update(@Valid @RequestBody TableCategory tableCategory) {
		return R.status(tableCategoryService.updateById(tableCategory));
	}

	/**
	 * 新增或修改 表分类
	 */
	@PostMapping("/submit")
	@ApiOperation(value = "新增或修改 submit", notes = "传入tableCategory")
	public R submit(@Valid @RequestBody TableCategory tableCategory) {
		return R.status(tableCategoryService.saveOrUpdate(tableCategory));
	}


	/**
	 * 删除 表分类
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(tableCategoryService.removeByIds(Func.toLongList(ids)));
	}

	/**
	 * 表绑定分类
	 */
	@PostMapping("/bind")
	@ApiOperation(value = "表绑定分类 bind", notes = "")
	public R bind(@ApiParam(value = "表绑定分类数据", required = true) @RequestBody BindTableCategoryRequest bindTableCategoryRequest) {
		String tableViewName = bindTableCategoryRequest.getTableViewName();
		Integer orderIndex = bindTableCategoryRequest.getOrderIndex();
		Integer categoryId = bindTableCategoryRequest.getCategoryId();
		if (StrUtil.isBlank(tableViewName) || null == categoryId) {
			return R.fail("缺少必要入参");
		}
		return R.data(insertOrUpdate(bindTableCategoryRequest, tableViewName, orderIndex, categoryId));

	}

	/**
	 * 修改表分类或者中文注释
	 * @param bindTableCategoryRequest
	 * @return
	 */
	@PostMapping("/update/category-component")
	@ApiOperation(value = "修改表分类或者中文注释", notes = "")
	public R updateCategoryAndComponent(@ApiParam(value = "表绑定分类数据", required = true) @RequestBody BindTableCategoryRequest bindTableCategoryRequest){
		boolean bool = true;
		String tableViewName = bindTableCategoryRequest.getTableViewName();
		Integer orderIndex = bindTableCategoryRequest.getOrderIndex();
		Integer categoryId = bindTableCategoryRequest.getCategoryId();
//		Integer datasourceId = bindTableCategoryRequest.getDatasourceId();
//		String tableComments = bindTableCategoryRequest.getTableComments();
		if (StrUtil.isBlank(tableViewName)) {
			return R.fail("表或者视图名称不能为空");
		}
		if (categoryId != null){
			bool = insertOrUpdate(bindTableCategoryRequest, tableViewName, orderIndex, categoryId);
		}
		if (!bool){
			return R.fail("修改表分类失败");
		}
//		//不要去修改物理表的注释
//		if (StrUtil.isNotEmpty(tableComments)){
//			if (datasourceId == null){
//				return R.fail("数据源id不能为空");
//			}
//			bool = updateTableComments(tableViewName,datasourceId,tableComments);
//		}
		if (!bool){
			return R.fail("修改表中文名失败");
		}

		return R.data(true);
	}

	/**
	 * 修改表中文注释
	 * @param tableViewName
	 * @param datasourceId
	 * @param tableComments
	 * @return
	 */
	private boolean updateTableComments(String tableViewName, Integer datasourceId, String tableComments) {
		String sqlStatement = "alter table " + tableViewName + " comment '" + tableComments + "'";
		ReportDatasource reportDatasource = reportDatasourceService.getById(datasourceId);
		if(reportDatasource == null) {
			throw new ServiceException("数据源不存在");
		}
		// 0-mysql 1-oracle 2-sqlserver 3-postgresql 4-clickhouse 5-kingbase
		if (reportDatasource.getDsType() == 1 || reportDatasource.getDsType() == 5 || reportDatasource.getDsType() == 3){
			sqlStatement = "comment on table" + tableViewName +" is '" + tableComments + "'";
		} else if (reportDatasource.getDsType() == 2) {
			sqlStatement = "execute sp_updateextendedproperty 'MS_Description','" +tableComments+"','user','dbo','table','"+tableViewName+ "',null,null";
		} else if (reportDatasource.getDsType() == 4) {

		}

		dynamicTableService.executeSql(reportDatasource,sqlStatement);
		return true;
	}

	/**
	 * 插入或更新表的分类
	 * @param bindTableCategoryRequest
	 * @param tableViewName
	 * @param orderIndex
	 * @param categoryId
	 * @return
	 */
	private boolean insertOrUpdate(BindTableCategoryRequest bindTableCategoryRequest, String tableViewName, Integer orderIndex, Integer categoryId) {
		//检查是否已存在
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("category_id", categoryId);
		queryWrapper.eq("table_view_name", tableViewName);
		List<TableCategoryRelation> list = tableCategoryRelationService.list(queryWrapper);
		if (null == list || list.size() == 0) {
			TableCategoryRelation tableCategoryRelation = new TableCategoryRelation();
			tableCategoryRelation.setCategoryId(categoryId);
			tableCategoryRelation.setOrderIndex(orderIndex);
			tableCategoryRelation.setTableViewName(tableViewName);
			tableCategoryRelation.setTableComments(bindTableCategoryRequest.getTableComments());
			tableCategoryRelation.setDatasourceId(bindTableCategoryRequest.getDatasourceId());
			return tableCategoryRelationService.save(tableCategoryRelation);
		} else {
			TableCategoryRelation tableCategoryRelation = list.get(0);
			tableCategoryRelation.setCategoryId(categoryId);
			tableCategoryRelation.setOrderIndex(orderIndex);
			tableCategoryRelation.setTableViewName(tableViewName);
			tableCategoryRelation.setTableComments(bindTableCategoryRequest.getTableComments());
			tableCategoryRelation.setDatasourceId(bindTableCategoryRequest.getDatasourceId());
			return tableCategoryRelationService.update(tableCategoryRelation, queryWrapper);
		}
	}


	/**
	 * 根据CategoryId查询表列表
	 */
	@GetMapping("/listByCategoryId")
	@ApiOperation(value = "根据CategoryId查询表列表", notes = "传入tableCategoryId")
	public R<List<TableCategoryDTO>> listByCategoryId(@RequestParam(value = "id", required = true) Integer id) {
		QueryWrapper queryWrapper = new QueryWrapper();
		if (null != id) {
			queryWrapper.eq("category_id", id);
		}
		queryWrapper.orderByDesc("order_index");
		List<TableCategoryRelation> list = tableCategoryRelationService.list(queryWrapper);

		//过滤需要权限访问的表
		TableCategory tableCategory = tableCategoryService.getById(id);
//		List<TableCategoryRelation> removeList = new ArrayList<>();
//		Long userId = SecureUtil.getUserId();
//		ReportDatasourceTablePerm tablePerm = reportDatasourceTablePermService.getOne(new LambdaQueryWrapper<ReportDatasourceTablePerm>()
//			.eq(ReportDatasourceTablePerm::getUserId,userId)
//			.eq(ReportDatasourceTablePerm::getDatasourceId,tableCategory.getDatasourceId())
//			.last("order by length(table_names) DESC limit 1")
//		);
//		if(list != null && tablePerm != null) {
//			if(tablePerm.getTableNames().length() == 0){
//				return R.data(null);
//			}else {
//				String[] split = tablePerm.getTableNames().split(",");
//				HashSet<String> set = new HashSet<>(Arrays.asList(split));
//				for (TableCategoryRelation reportViewDTO : list) {
//					boolean contains = set.contains(reportViewDTO.getTableViewName());
//					if(!contains){
//						removeList.add(reportViewDTO);
//					}
//				}
//			}
//			if(removeList.size() > 0){
//				list.removeAll(removeList);
//			}
//		}

		List<TableCategoryDTO> data = new ArrayList<>();
		if (null != list && list.size() > 0) {
			list.forEach(tableCategoryRelation -> {
				TableCategoryDTO dto = new TableCategoryDTO();
				//TableCategory tableCategory = tableCategoryService.getById(tableCategoryRelation.getCategoryId());
				if (null != tableCategory) {
					dto.setCategoryName(tableCategory.getCategoryName());
					dto.setDatasourceId(tableCategory.getDatasourceId());
				}
				dto.setCategoryId(tableCategoryRelation.getCategoryId());
				dto.setOrderIndex(tableCategoryRelation.getOrderIndex());
				dto.setTableViewName(tableCategoryRelation.getTableViewName());
				dto.setTableComments(tableCategoryRelation.getTableComments());

				data.add(dto);
			});
		}
		return R.data(data);
	}
}
