package com.ff.apimngr.jwt;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.ff.apimngr.entity.Role;
import com.ff.apimngr.entity.SystemLog;
import com.ff.apimngr.entity.User;
import com.ff.apimngr.service.SystemLogService;
import com.ff.apimngr.service.UserService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Component
public class CorsFilter implements Filter {


    @Resource
    private UserService userService;

    @Resource
    private SystemLogService systemLogService;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {

        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;
        String url = request.getServletPath();
        if (isWhiteListUrl(url)) {
            response.setHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));
            response.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
            response.setHeader("Access-Control-Max-Age", "3600");
            response.setHeader("Access-Control-Allow-Headers", "token");
            response.setHeader("Access-Control-Allow-Credentials", "true"); //是否支持cookie跨域
            filterChain.doFilter(servletRequest, servletResponse);
            return;
        }
        //filterChain.doFilter(servletRequest, servletResponse);//remove after testing

        String token = request.getHeader("token");
        String account = "";
        if (token != null && token.length() > 0) {
            account = JwtHelper.parseToken(token);
            if (isAdminUrl(url)) {
                User user = userService.findUserByAccount(account);
                if (null != user) {
                    Role role = user.getRole();
                    if (null != role) {
                        if (role.getRoleType() != 1) {
                            response.setHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));
                            response.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
                            response.setHeader("Access-Control-Max-Age", "3600");
                            response.setHeader("Access-Control-Allow-Headers", "token");
                            response.setHeader("Access-Control-Allow-Credentials", "true"); //是否支持cookie跨域
                            response.setHeader("Access-Control-Expose-Headers", "sessiontimeout");
                            response.setHeader("sessiontimeout", "true");
                            return;
                        }
                    }
                }
            }
        }
        //account = "admin"; for local testing

        if (null != account && account.length() > 0) {
            //记录日志
            SystemLog systemLog = new SystemLog();
            systemLog.setLogAccount(account);
            systemLog.setLogContent(request.getServletPath());
            //systemLog.setLogIp(request.getRemoteHost());
            systemLog.setLogIp(getRemoteIP(request));
            Date now = new Date();
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            systemLog.setLogTime(format.format(now));
            systemLogService.insertSystemLog(systemLog);

            response.setHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));
            response.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
            response.setHeader("Access-Control-Max-Age", "3600");
            response.setHeader("Access-Control-Allow-Headers", "token");
            response.setHeader("Access-Control-Allow-Credentials", "true"); //是否支持cookie跨域
            filterChain.doFilter(servletRequest, servletResponse);

        } else {
            response.setHeader("Access-Control-Allow-Origin", request.getHeader("Origin"));
            response.setHeader("Access-Control-Allow-Methods", "POST, GET, OPTIONS, DELETE");
            response.setHeader("Access-Control-Max-Age", "3600");
            response.setHeader("Access-Control-Allow-Headers", "token");
            response.setHeader("Access-Control-Allow-Credentials", "true"); //是否支持cookie跨域
            response.setHeader("Access-Control-Expose-Headers", "sessiontimeout");
            response.setHeader("sessiontimeout", "true");
        }


    }

    private boolean isAdminUrl(String url) {
        if (url.equals("/api/user/list") || url.equals("/api/role/list") || url.equals("/api/codeinfo/list")
                || url.equals("/api/systemlog/list")){
            return true;
        } else {
            return false;
        }

    }

    private boolean isWhiteListUrl(String url) {
        if (url.equals("/api/user/login") || url.equals("/api/user/logout") || url.equals("/api/user/getverify") || url.equals("/api/user/insert") || url.equals("/api/redis/test")
                || url.equals("/api/user/gencode") || url.equals("/api/user/captchaImage")
                || url.equals("/api/demo/compare") || url.equals("/api/demo/detect") || url.equals("/api/demo/search") || url.equals("/api/demo/addface")
                || url.equals("/api/demo/getSessionCode") || url.equals("/api/demo/hVideo")
                || url.equals("/api/user/getsms") || url.equals("/api/user/getpwsms") || url.equals("/api/user/resetpw") || url.equals("/api/file/uploadfile")
                || url.equals("/api/file/getimg") || url.equals("/api/device/uploadfile") || url.equals("/api/sms/sendbypassword") || url.equals("/api/user/checkvalidcode")
                || url.equals("/api/sms/resetpassword") || url.equals("/api/file/uploadmusic") || url.equals("/api/file/getmusic")
                || url.startsWith("/swagger") || url.startsWith("/webjars/springfox-swagger-ui") || url.equals("/v2/api-docs")
                || url.equals("/druid") || url.equals("/api/apimngr/insertNoAuth")  || url.equals("/api/apitype/list")
                || url.startsWith("/api/homepage") || url.startsWith("/api/open")) {
            return true;
        } else {
            return false;
        }

    }

    public String convertObjectToJson(Object object) throws JsonProcessingException {
        if (object == null) {
            return null;
        }
        ObjectMapper mapper = new ObjectMapper();
        return mapper.writeValueAsString(object);
    }

    @Override
    public void destroy() {

    }

    public String getRemoteIP(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Real-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

}
